import logging

logger = logging.getLogger(f"netbox_config_backup")


def get_backup_tables(instance):
    from netbox_config_backup.models import BackupCommitTreeChange
    from netbox_config_backup.tables import BackupsTable

    def get_backup_table(data):
        backups = []
        for row in data:
            commit = row.commit
            current = row
            previous = row.backup.changes.filter(file__type=row.file.type, commit__time__lt=commit.time).last()
            backup = {'pk': instance.pk, 'date': commit.time, 'current': current, 'previous': previous}
            backups.append(backup)

        table = BackupsTable(backups)
        return table

    backups = BackupCommitTreeChange.objects.filter(backup=instance).prefetch_related('backup', 'new', 'old', 'commit',
                                                                                      'file').order_by('commit__time')

    tables = {}
    for file in ['running', 'startup']:
        try:
            tables.update({file: get_backup_table(backups.filter(file__type=file))})
        except KeyError:
            tables.update({file: get_backup_table([])})

    return tables



from utilities.choices import ChoiceSet


#
# File Types for Backup Files
#

class FileTypeChoices(ChoiceSet):

    TYPE_RUNNING = 'running'
    TYPE_STARTUP = 'startup'

    CHOICES = (
        (TYPE_RUNNING, 'Running'),
        (TYPE_STARTUP, 'Startup'),
    )

class CommitTreeChangeTypeChoices(ChoiceSet):

    TYPE_ADD = 'add'
    TYPE_MODIFY = 'modify'

    CHOICES = (
        (TYPE_ADD, 'Add'),
        (TYPE_MODIFY, 'Modify'),
    )


class StatusChoices(ChoiceSet):
    STATUS_ACTIVE = 'active'
    STATUS_DISABLED = 'disabled'

    CHOICES = (
        (STATUS_ACTIVE, 'Active'),
        (STATUS_DISABLED, 'Disabled'),
    )



# Generated by Django 4.0.3 on 2022-04-07 03:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ipam', '0057_created_datetimefield'),
        ('netbox_config_backup', '0009_bctc_file_model_backup_fk_restructure'),
    ]

    operations = [
        migrations.AddField(
            model_name='backup',
            name='ip',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ipam.ipaddress'),
        ),
    ]



from django.db import models
from django.urls import reverse

from netbox_config_backup.choices import FileTypeChoices
from netbox_config_backup.models import Backup
from netbox_config_backup.models.abstract import BigIDModel


class BackupCommit(BigIDModel):
    sha = models.CharField(max_length=64)
    time = models.DateTimeField()

    def __str__(self):
        return self.sha


class BackupObject(BigIDModel):
    sha = models.CharField(max_length=64, unique=True)

    def __str__(self):
        return f'{self.sha}'


class BackupFile(BigIDModel):
    backup = models.ForeignKey(to=Backup, on_delete=models.CASCADE, null=False, blank=False, related_name='files')
    type = models.CharField(max_length=10, choices=FileTypeChoices, null=False, blank=False)

    class Meta:
        unique_together = ['backup', 'type']

    def __str__(self):
        return f'{self.name}.{self.type}'

    @property
    def name(self):
        return f'{self.backup.uuid}'

    @property
    def path(self):
        return f'{self.name}.{self.type}'


class BackupCommitTreeChange(BigIDModel):
    backup = models.ForeignKey(to=Backup, on_delete=models.CASCADE, null=False, blank=False, related_name='changes')
    file = models.ForeignKey(to=BackupFile, on_delete=models.CASCADE, null=False, blank=False, related_name='changes')

    commit = models.ForeignKey(to=BackupCommit, on_delete=models.PROTECT, related_name='changes')
    type = models.CharField(max_length=10)
    old = models.ForeignKey(to=BackupObject, on_delete=models.PROTECT, related_name='previous', null=True)
    new = models.ForeignKey(to=BackupObject, on_delete=models.PROTECT, related_name='changes', null=True)

    def __str__(self):
        return f'{self.commit.sha}-{self.type}'

    def filename(self):
        return f'{self.backup.uuid}.{self.type}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_config_backup:backup_config', kwargs={'backup': self.backup.pk, 'current': self.pk})

    @property
    def previous(self):
        return self.backup.changes.filter(file__type=self.file.type, commit__time__lt=self.commit.time).last()


import os
import difflib
from datetime import datetime
from time import sleep

from deepdiff import DeepDiff
from dulwich import repo, porcelain, object_store
from pydriller import Git

from netbox import settings

__all__ = (
    'repository'
)

from netbox_config_backup.helpers import get_repository_dir


def encode(value, encoding):
    if value is not None:
        return value.encode(encoding)
    return None


def decode(value, encoding):
    if value is not None:
        return value.decode(encoding)
    return None


class GitBackup:
    repository = None
    driller = None
    location = None

    def __init__(self):
        self.location = get_repository_dir()

        if os.path.exists(self.location):
            self.repository = repo.Repo(self.location)

        if self.repository is None:
            self.repository = repo.Repo.init(path=self.location, mkdir=True)

        if self.repository is not None:
            try:
                self.driller = Git(self.location)
            except OSError:
                pass

    def write(self, file, data):
        path = f'{self.location}{os.path.sep}{file}'
        with open(path, 'w') as f:
            f.write(data)
            f.close()

        failures = 0
        while failures < 10:
            try:
                porcelain.add(self.repository, path)
                return
            except FileExistsError:
                sleep(1)
                failures = failures + 1
                if failures >= 10:
                    raise Exception('Unable to acquire lock on repository in a timely manner')


    def commit(self, message):
        committer = settings.PLUGINS_CONFIG.get('netbox_config_backup', {}).get('committer', None)
        author = settings.PLUGINS_CONFIG.get('netbox_config_backup', {}).get('author', None)

        if author is not None:
            author = author.encode('ascii')
        if committer is not None:
            committer = committer.encode('ascii')

        failures = 0
        while failures < 10:
            try:
                commit = porcelain.commit(self.repository, message, committer=committer, author=author)
                return commit.decode('ascii')
            except repo.InvalidUserIdentity:
                committer = 'Your NetBox is misconfigured <netbox@localhost>'.encode('ascii')
                author = 'Your NetBox is misconfigured <netbox@localhost>'.encode('ascii')
            except FileExistsError:
                sleep(1)
                failures = failures + 1
                if failures >= 10:
                    raise Exception('Unable to acquire lock on repository in a timely manner')

    def read(self, file, index=None):
        path = file.encode('ascii')
        if index is None:
            index = 'HEAD'

        try:
            tree = self.repository[index.encode('ascii')].tree
            mode, sha = object_store.tree_lookup_path(self.repository.__getitem__, tree, path)
            data = self.repository[sha].data.decode('ascii')
            return data
        except KeyError:
            return None

    def diff(self, file, a=None, b=None):
        path = file.encode('ascii')
        commits = [a, b]
        data = []
        for commit in commits:
            if commit is None:
                data.append(None)
            else:
                data.append(self.read(file, commit))

        diff = DeepDiff(data[0], data[1]).diff()
        return diff

    def log(self, file=None, paths=[], index=None, depth=None):
        def get_index(haystack, needle):
            key = f'{needle.file}'

        if file is not None:
            path = file.encode('ascii')
            paths = [path]
        else:
            path = None
            for idx in range(0, len(paths)):
                path = paths[idx]
                paths[idx] = path.encode('ascii')

        if index is not None:
            index = index.encode('ascii')

        walker = self.repository.get_walker(include=index, paths=paths, max_entries=depth)
        entries = [entry for entry in walker]

        indexes = []
        for entry in entries:
            output = {}
            encoding = entry.commit.encoding.decode('ascii') if entry.commit.encoding else 'ascii'
            parents = []
            output = {
                'author': decode(entry.commit.author, encoding),
                'committer': decode(entry.commit.committer, encoding),
                'message': decode(entry.commit.message, encoding),
                'parents': [decode(parent, encoding) for parent in entry.commit.parents],
                'sha': str(entry.commit.sha().hexdigest()),
                'time': datetime.fromtimestamp(entry.commit.commit_time),
                'tree': decode(entry.commit.tree, encoding)
            }
            changes = []
            for change in entry.changes():
                old = {'sha': decode(change.old.sha, encoding), 'path': decode(change.old.path, encoding)}
                if path is not None and (change.old.path == path or change.new.path == path):
                    output.update({
                        'change': {
                            'type': change.type,
                            'old': {
                                'path': old.get('path'),
                                'sha': old.get('sha'),
                            },
                            'new': {
                                'path': decode(change.new.path, encoding),
                                'sha': decode(change.new.sha, encoding)
                            },
                        }
                    })
                changes.append({
                    'type': change.type,
                    'old': {
                        'path': decode(change.old.path, encoding),
                        'sha': decode(change.old.sha, encoding)
                    },
                    'new': {
                        'path': decode(change.new.path, encoding),
                        'sha': decode(change.new.sha, encoding)
                    },
                })

            output.update({'changes': changes})
            indexes.append(output)

        return indexes


repository = GitBackup()



from rest_framework.viewsets import ModelViewSet

from netbox_config_backup.api import BackupSerializer
from netbox_config_backup.models import Backup


class BackupViewSet(ModelViewSet):
    queryset = Backup.objects.all()
    serializer_class = BackupSerializer


import logging

from django.db import models
from django.db.models import ForeignKey
from django.utils import timezone
from django_rq import get_queue
from core.choices import JobStatusChoices
from .abstract import BigIDModel

logger = logging.getLogger(f"netbox_config_backup")


class BackupJob(BigIDModel):
    backup = ForeignKey(
        to='Backup',
        on_delete=models.CASCADE,
        blank=False,
        null=False,
        related_name='jobs',
    )
    created = models.DateTimeField(
        auto_now_add=True
    )
    scheduled = models.DateTimeField(
        null=True,
        blank=True
    )
    started = models.DateTimeField(
        null=True,
        blank=True
    )
    completed = models.DateTimeField(
        null=True,
        blank=True
    )
    status = models.CharField(
        max_length=30,
        choices=JobStatusChoices,
        default=JobStatusChoices.STATUS_PENDING
    )
    data = models.JSONField(
        null=True,
        blank=True
    )
    job_id = models.UUIDField(
        unique=True
    )

    def __str__(self):
        return str(self.job_id)

    def delete(self, using=None, keep_parents=False):
        queue = get_queue('netbox_config_backup.jobs')

        job = queue.fetch_job(f'{self.job_id}')
        if job is not None:
            job.cancel()
            job.remove()

        super().delete(using=using, keep_parents=keep_parents)

    @property
    def queue(self):
        return get_queue('netbox_config_backup.jobs')

    @property
    def duration(self):
        if not self.completed:
            return None

        duration = self.completed - self.started
        minutes, seconds = divmod(duration.total_seconds(), 60)

        return f"{int(minutes)} minutes, {seconds:.2f} seconds"

    def set_status(self, status):
        """
        Helper method to change the status of the job result. If the target status is terminal, the  completion
        time is also set.
        """
        self.status = status
        if status in JobStatusChoices.TERMINAL_STATE_CHOICES:
            self.completed = timezone.now()

    def reschedule(self, time):
        """
        Reschedule a job
        """
        if self.status == JobStatusChoices.STATUS_PENDING:
            self.scheduled = time
            job = self.queue.fetch_job(f'{self.job_id}')
            self.queue.schedule(job, time)
        else:
            raise Exception('Job is not in a state for rescheduling')

    @classmethod
    def enqueue(cls, backup, delay=None):
        from netbox_config_backup.utils import enqueue
        return enqueue(backup, delay)

    @classmethod
    def enqueue_if_needed(cls, backup, delay=None, job_id=None):
        from netbox_config_backup.utils import enqueue_if_needed
        return enqueue_if_needed(backup, delay, job_id)

    @classmethod
    def needs_enqueue(cls, backup, job_id=None):
        from netbox_config_backup.utils import needs_enqueue
        return needs_enqueue(backup, job_id)

    @classmethod
    def is_running(cls, backup, job_id=None):
        from netbox_config_backup.utils import is_running
        return is_running(backup, job_id)

    @classmethod
    def is_queued(cls, backup, job_id=None):
        from netbox_config_backup.utils import is_queued
        return is_queued(backup, job_id)

    @classmethod
    def remove_orphaned(cls):
        from netbox_config_backup.utils import remove_orphaned
        return remove_orphaned()

    @classmethod
    def remove_queued(cls, backup):
        from netbox_config_backup.utils import remove_queued
        return remove_queued()


from django.core.exceptions import ValidationError
from django.test import TestCase

from dcim.models import Site, Manufacturer, DeviceType, DeviceRole, Device, Interface
from ipam.models import IPAddress
from netbox_config_backup.models import *

class TestBackup(TestCase):

    @classmethod
    def setUpTestData(cls):

        site = Site.objects.create(name='Site 1')
        manufacturer = Manufacturer.objects.create(name='Manufacturer 1')
        device_type = DeviceType.objects.create(model='Device Type 1', manufacturer=manufacturer)
        role = DeviceRole.objects.create(name='Switch')
        device = Device.objects.create(
            name='Device 1',
            site=site,
            device_type=device_type,
            role=role,
            status='active'
        )
        interface = Interface.objects.create(name='Interface 1', device=device, type='1000baset')
        address = IPAddress.objects.create(assigned_object=interface, address='10.0.0.1/32')
        device.primary_ip4 = address
        device.save()

    def test_create_backup(self):
        pass



import django_tables2 as tables
from django_tables2.utils import Accessor

from netbox_config_backup.models import Backup, BackupCommitTreeChange
from netbox.tables import columns, BaseTable, NetBoxTable


class ActionButtonsColumn(tables.TemplateColumn):
    attrs = {'td': {'class': 'text-end text-nowrap noprint min-width'}}
    template_code = """
    <a href="{% url 'plugins:netbox_config_backup:backup_config' backup=record.backup.pk current=record.pk %}" class="btn btn-sm btn-outline-dark" title="View">
        <i class="mdi mdi-cloud-download"></i>
    </a>
    {% if record.previous %}
        <a href="{% url 'plugins:netbox_config_backup:backup_diff' backup=record.backup.pk current=record.pk previous=record.previous.pk %}" class="btn btn-outline-dark btn-sm" title="Diff">
            <i class="mdi mdi-file-compare"></i>
        </a>
    {% else %}
        
    {% endif %}
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, template_code=self.template_code, **kwargs)

    def header(self):
        return ''


class BackupTable(BaseTable):
    pk = columns.ToggleColumn(

    )
    name = tables.Column(
        linkify=True,
        verbose_name='Backup Name'
    )
    device = tables.Column(
        linkify={
            'viewname': 'dcim:device',
            'args': [Accessor('device_id')],
        }
    )
    last_backup = tables.DateTimeColumn()
    next_attempt = tables.DateTimeColumn()
    last_change = tables.DateTimeColumn()
    backup_count = tables.Column(
        accessor='changes'
    )

    class Meta(BaseTable.Meta):
        model = Backup
        fields = (
            'pk', 'name', 'device', 'last_backup', 'next_attempt', 'last_change', 'backup_count'
        )
        default_columns = (
            'pk', 'name', 'device', 'last_backup', 'next_attempt', 'backup_count'
        )

    def render_backup_count(self, value):
        return f'{value.count()}'


class BackupsTable(NetBoxTable):
    date = tables.Column(
        accessor='commit__time'
    )
    type = tables.Column(
        accessor='file__type'
    )
    actions = ActionButtonsColumn()

    class Meta:
        model = BackupCommitTreeChange
        fields = (
            'pk', 'id', 'date', 'type', 'backup', 'commit', 'file', 'actions'
        )
        default_columns = (
            'pk', 'id', 'date', 'type', 'actions'
        )
        attrs = {
            'class': 'table table-hover object-list',
        }
        order_by = ['-date']



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        from netbox_config_backup.models import Backup, BackupJob

        for backup in Backup.objects.all():
            if BackupJob.needs_enqueue(backup=backup):
                BackupJob.enqueue_if_needed(backup=backup)
                print(f'Backup: {backup} has been queued')




import uuid

from django.core.management.base import BaseCommand
from django.db import transaction
from django.utils import timezone

from netbox_config_backup.models import BackupJob
from netbox_config_backup.tasks import backup_job
from netbox_config_backup.utils import remove_queued
from netbox_config_backup.utils.rq import can_backup


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--time', dest='time', help="time")
        parser.add_argument('--device', dest='device', help="Device Name")

    def handle(self, *args, **options):
        from multiprocessing import Process
        import time
        def test(i):
            self.stdout.write(f"Child {i} is running")
            self.stdout.write(f"Child {i} sleeping 10 seconds")
            time.sleep(10)
            self.stdout.write(f"Child {i} sleep complete")

        processes = []
        for i in range(1, 2):
            p = Process(target=test, args=(i,))
            p.start()
            p.join(1)
            self.stdout.write(f"Child {i} running")
            processes.append(p)




import difflib
import re
import logging
from deepdiff import DeepDiff

logger = logging.getLogger(f"netbox_config_backup")


class Differ(DeepDiff):
    def is_diff(self):
        if self.get('values_changed'):
            return True
        return False

    def diff(self):
        diff = self.get('values_changed', {}).get('root', {}).get('diff', '')
        return diff

    def compare(self):
        diff = self.diff()
        return diff.splitlines()

    def cisco_compare(self):
        return self.compare()



from django.db import models
from django.db.models import Count

from core.choices import JobStatusChoices
from utilities.querysets import RestrictedQuerySet



class BackupQuerySet(RestrictedQuerySet):
    def default_annotate(self):
        from netbox_config_backup.models import BackupJob, BackupCommitTreeChange

        return self.annotate(
            last_backup=models.Subquery(
                BackupJob.objects.filter(
                    backup=models.OuterRef('id'),
                    status=JobStatusChoices.STATUS_COMPLETED).order_by('-completed').values('completed')[:1]
            ),
            next_attempt=models.Subquery(
                BackupJob.objects.filter(
                    backup=models.OuterRef('id'),
                    status__in=[
                        JobStatusChoices.STATUS_PENDING, JobStatusChoices.STATUS_RUNNING
                    ]
                ).order_by('-scheduled').values('scheduled')[:1]
            ),
            last_change=models.Subquery(
                BackupCommitTreeChange.objects.filter(
                    backup=models.OuterRef('id')
                ).order_by('-id').values('commit__time')[:1]
            )
        )



import logging

import uuid
from django.utils import timezone
from django_rq import get_queue
from rq.registry import ScheduledJobRegistry

from dcim.choices import DeviceStatusChoices
from core.choices import JobStatusChoices
from netbox_config_backup.choices import StatusChoices
from netbox_config_backup.models.jobs import BackupJob

logger = logging.getLogger(f"netbox_config_backup")


def can_backup(backup):
    logger.debug(f'Checking backup status for {backup}')
    if backup.device is None:
        logger.info(f'No device for {backup}')
        return False
    elif backup.status == StatusChoices.STATUS_DISABLED:
        logger.info(f'Backup disabled for {backup}')
        return False
    elif backup.device.status in [DeviceStatusChoices.STATUS_OFFLINE,
                                DeviceStatusChoices.STATUS_FAILED,
                                DeviceStatusChoices.STATUS_INVENTORY,
                                DeviceStatusChoices.STATUS_PLANNED]:
        logger.info(f'Backup disabled for {backup} due to device status ({backup.device.status})')
        return False
    elif (backup.ip is None and backup.device.primary_ip is None) or backup.device.platform is None or \
            hasattr(backup.device.platform, 'napalm') is False or backup.device.platform.napalm is None or \
            backup.device.platform.napalm.napalm_driver == '' or backup.device.platform.napalm.napalm_driver is None:
        if backup.ip is None and backup.device.primary_ip is None:
            logger.warning(f'Backup disabled for {backup} due to no primary IP ({backup.device.status})')
        elif backup.device.platform is None:
            logger.warning(f'Backup disabled for {backup} due to platform not set ({backup.device.status})')
        elif hasattr(backup.device.platform, 'napalm') is False or backup.device.platform.napalm is None:
            logger.warning(
                f'Backup disabled for {backup} due to platform having no napalm config ({backup.device.status})'
            )
        elif backup.device.platform.napalm.napalm_driver == '' or backup.device.platform.napalm.napalm_driver is None:
            logger.warning(f'Backup disabled for {backup} due to napalm driver not set ({backup.device.status})')
        return False

    return True

def enqueue(backup, delay=None):
    from netbox_config_backup.models import BackupJob

    scheduled = timezone.now()
    if delay is not None:
        logger.info(f'{backup}: Scheduling for: {scheduled + delay} at {scheduled} ')
        scheduled = timezone.now() + delay

    result = BackupJob.objects.create(
        backup=backup,
        job_id=uuid.uuid4(),
        scheduled=scheduled,
    )
    queue = get_queue('netbox_config_backup.jobs')
    if delay is None:
        logger.debug(f'{backup}: Enqueued')
        job = queue.enqueue(
            'netbox_config_backup.tasks.backup_job',
            description=f'{backup.uuid}',
            job_id=str(result.job_id),
            pk=result.pk,
        )
        logger.info(f'{backup}: {result.job_id}')
    else:
        logger.debug(f'{backup}: Enqueued')
        job = queue.enqueue_in(
            delay,
            'netbox_config_backup.tasks.backup_job',
            description=f'{backup.uuid}',
            job_id=str(result.job_id),
            pk=result.pk
        )
        logger.info(f'{backup}: {result.job_id}')

    return job


def enqueue_if_needed(backup, delay=None, job_id=None):
    if needs_enqueue(backup, job_id=job_id):
        return enqueue(backup, delay=delay)
    return False


def needs_enqueue(backup, job_id=None):
    queue = get_queue('netbox_config_backup.jobs')
    scheduled = queue.scheduled_job_registry
    started = queue.started_job_registry

    scheduled_jobs = scheduled.get_job_ids()
    started_jobs = started.get_job_ids()

    if not can_backup(backup):
        return False
    elif is_queued(backup, job_id):
        logger.info(f'Backup already queued for {backup}')
        return False

    return True


def is_running(backup, job_id=None):
    queue = get_queue('netbox_config_backup.jobs')

    jobs = backup.jobs.all()
    queued = jobs.filter(status__in=[JobStatusChoices.STATUS_RUNNING])

    if job_id is not None:
        queued.exclude(job_id=job_id)

    for backupjob in queued.all():
        job = queue.fetch_job(f'{backupjob.job_id}')
        if job and job.is_started and job.id in queue.started_job_registry.get_job_ids() + queue.get_job_ids():
            return True
        elif job and job.is_started and job.id not in queue.started_job_registry.get_job_ids():
            status = {
                     'is_canceled': job.is_canceled,
                     'is_deferred': job.is_deferred,
                     'is_failed': job.is_failed,
                     'is_finished': job.is_finished,
                     'is_queued': job.is_queued,
                     'is_scheduled': job.is_scheduled,
                     'is_started': job.is_started,
                     'is_stopped': job.is_stopped,
            }
            job.cancel()
            backupjob.status = JobStatusChoices.STATUS_FAILED
            backupjob.save()
            logger.warning(f'{backup}: Job in started queue but not in a registry, cancelling {status}')
        elif job and job.is_canceled:
            backupjob.status = JobStatusChoices.STATUS_FAILED
            backupjob.save()
    return False


def get_scheduled(backup, job_id=None):
    queue = get_queue('netbox_config_backup.jobs')

    scheduled_jobs = queue.scheduled_job_registry.get_job_ids()
    started_jobs = queue.started_job_registry.get_job_ids()
    queued_jobs = queue.get_job_ids()

    jobs = backup.jobs.all()
    queued = jobs.filter(status__in=[JobStatusChoices.STATUS_RUNNING, JobStatusChoices.STATUS_PENDING])

    if job_id is not None:
        queued.exclude(job_id=job_id)

    for backupjob in queued.all():
        job = queue.fetch_job(f'{backupjob.job_id}')
        if job and (job.is_scheduled or job.is_queued) and job.id in scheduled_jobs + started_jobs + queued_jobs:
            if job.enqueued_at is not None:
                return job.enqueued_at
            else:
                return queue.scheduled_job_registry.get_scheduled_time(job)
        elif job and (job.is_scheduled or job.is_queued) and job.id not in scheduled_jobs + started_jobs:
            status = {
                     'is_canceled': job.is_canceled,
                     'is_deferred': job.is_deferred,
                     'is_failed': job.is_failed,
                     'is_finished': job.is_finished,
                     'is_queued': job.is_queued,
                     'is_scheduled': job.is_scheduled,
                     'is_started': job.is_started,
                     'is_stopped': job.is_stopped,
            }
            job.cancel()
            backupjob.status = JobStatusChoices.STATUS_FAILED
            backupjob.save()
            logger.warning(f'{backup}: Job in scheduled or started queue but not in a registry, cancelling {status} {scheduled_jobs + started_jobs + queued_jobs}')
        elif job and job.is_canceled:
            backupjob.status = JobStatusChoices.STATUS_FAILED
            backupjob.save()
    return None


def is_queued(backup, job_id=None):
    if get_scheduled(backup, job_id) is not None:
        return True
    return False


def remove_orphaned():
    queue = get_queue('netbox_config_backup.jobs')
    registry = ScheduledJobRegistry(queue=queue)

    for job_id in registry.get_job_ids():
        try:
            BackupJob.objects.get(job_id=job_id)
        except BackupJob.DoesNotExist:
            registry.remove(job_id)


def remove_queued(backup):
    queue = get_queue('netbox_config_backup.jobs')
    registry = ScheduledJobRegistry(queue=queue)
    for job_id in registry.get_job_ids():
        job = queue.fetch_job(f'{job_id}')
        if job.description == f'{backup.uuid}':
            registry.remove(f'{job_id}')



from rest_framework import serializers

from netbox.api.serializers import WritableNestedSerializer
from netbox_config_backup.models import Backup


__all__ = (
    'NestedBackupSerializer',
)


class NestedBackupSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_config_backup-api:backup-detail')

    class Meta:
        model = Backup
        fields = [
            'id', 'url', 'display', 'status'
        ]
